# Stubs for email.generator (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Generator:
    maxheaderlen = ...  # type: Any
    policy = ...  # type: Any
    def __init__(self, outfp, mangle_from_=..., maxheaderlen=..., *, policy=...) -> None: ...
    def write(self, s): ...
    def flatten(self, msg, unixfrom=..., linesep=...): ...
    def clone(self, fp): ...

class BytesGenerator(Generator):
    def write(self, s): ...

class DecodedGenerator(Generator):
    def __init__(self, outfp, mangle_from_=..., maxheaderlen=..., fmt=...) -> None: ...
