# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib_response (Python 2.7)

from six.moves.urllib.response import (
    addbase as addbase,
    addclosehook as addclosehook,
    addinfo as addinfo,
    addinfourl as addinfourl,
)
