# Stubs for backports_abc (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def mk_gen(): ...
def mk_awaitable(): ...
def mk_coroutine(): ...

Generator = ... # type: Any
Awaitable = ... # type: Any
Coroutine = ... # type: Any

def isawaitable(obj): ...

PATCHED = ... # type: Any

def patch(patch_inspect=True): ...
