# Stubs for concurrent.futures.thread (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from . import _base

class _WorkItem:
    future = ... # type: Any
    fn = ... # type: Any
    args = ... # type: Any
    kwargs = ... # type: Any
    def __init__(self, future, fn, args, kwargs): ...
    def run(self): ...

class ThreadPoolExecutor(_base.Executor):
    def __init__(self, max_workers=None): ...
    def submit(self, fn, *args, **kwargs): ...
    def shutdown(self, wait=True): ...
