# Stubs for http.cookies (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class CookieError(Exception): ...

class Morsel(dict):
    def __init__(self): ...
    @property
    def key(self): ...
    @key.setter
    def key(self, key): ...
    @property
    def value(self): ...
    @value.setter
    def value(self, value): ...
    @property
    def coded_value(self): ...
    @coded_value.setter
    def coded_value(self, coded_value): ...
    def __setitem__(self, K, V): ...
    def setdefault(self, key, val=None): ...
    def __eq__(self, morsel): ...
    __ne__ = ... # type: Any
    def copy(self): ...
    def update(self, values): ...
    def isReservedKey(self, K): ...
    def set(self, key, val, coded_val, LegalChars=...): ...
    def output(self, attrs=None, header=''): ...
    def js_output(self, attrs=None): ...
    def OutputString(self, attrs=None): ...

class BaseCookie(dict):
    def value_decode(self, val): ...
    def value_encode(self, val): ...
    def __init__(self, input=None): ...
    def __setitem__(self, key, value): ...
    def output(self, attrs=None, header='', sep=''): ...
    def js_output(self, attrs=None): ...
    def load(self, rawdata): ...

class SimpleCookie(BaseCookie):
    def value_decode(self, val): ...
    def value_encode(self, val): ...
