# Stubs for thrift.transport (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Names in __all__ with no definition:
#   THttpClient
#   TSocket
#   TTransport
#   TZlibTransport
