# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib.response (Python 3.2)

from urllib.response import addbase as addbase
from urllib.response import addclosehook as addclosehook
from urllib.response import addinfo as addinfo
from urllib.response import addinfourl as addinfourl
