# Stubs for email.policy (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
import email._policybase

Policy = email._policybase.Policy
Compat32 = email._policybase.Compat32

class EmailPolicy(Policy):
    refold_source = ...  # type: Any
    header_factory = ...  # type: Any
    content_manager = ...  # type: Any
    def __init__(self, **kw) -> None: ...
    def header_max_count(self, name): ...
    def header_source_parse(self, sourcelines): ...
    def header_store_parse(self, name, value): ...
    def header_fetch_parse(self, name, value): ...
    def fold(self, name, value): ...
    def fold_binary(self, name, value): ...

default = ...  # type: Any
strict = ...  # type: Any
SMTP = ...  # type: Any
HTTP = ...  # type: Any
