# Stubs for boto (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
import logging

Version = ... # type: Any
UserAgent = ... # type: Any
config = ... # type: Any
BUCKET_NAME_RE = ... # type: Any
TOO_LONG_DNS_NAME_COMP = ... # type: Any
GENERATION_RE = ... # type: Any
VERSION_RE = ... # type: Any
ENDPOINTS_PATH = ... # type: Any

def init_logging(): ...

class NullHandler(logging.Handler):
    def emit(self, record): ...

log = ... # type: Any
perflog = ... # type: Any

def set_file_logger(name, filepath, level=..., format_string=...): ...
def set_stream_logger(name, level=..., format_string=...): ...
def connect_sqs(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_s3(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_gs(gs_access_key_id=..., gs_secret_access_key=..., **kwargs): ...
def connect_ec2(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_elb(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_autoscale(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudwatch(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_sdb(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_fps(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_mturk(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudfront(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_vpc(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_rds(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_rds2(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_emr(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_sns(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_iam(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_route53(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudformation(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_euca(host=..., aws_access_key_id=..., aws_secret_access_key=..., port=..., path=..., is_secure=..., **kwargs): ...
def connect_glacier(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_ec2_endpoint(url, aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_walrus(host=..., aws_access_key_id=..., aws_secret_access_key=..., port=..., path=..., is_secure=..., **kwargs): ...
def connect_ses(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_sts(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_ia(ia_access_key_id=..., ia_secret_access_key=..., is_secure=..., **kwargs): ...
def connect_dynamodb(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_swf(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudsearch(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudsearch2(aws_access_key_id=..., aws_secret_access_key=..., sign_request=..., **kwargs): ...
def connect_cloudsearchdomain(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_beanstalk(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_elastictranscoder(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_opsworks(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_redshift(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_support(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudtrail(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_directconnect(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_kinesis(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_logs(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_route53domains(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cognito_identity(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cognito_sync(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_kms(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_awslambda(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_codedeploy(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_configservice(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_cloudhsm(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_ec2containerservice(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def connect_machinelearning(aws_access_key_id=..., aws_secret_access_key=..., **kwargs): ...
def storage_uri(uri_str, default_scheme=..., debug=..., validate=..., bucket_storage_uri_class=..., suppress_consec_slashes=..., is_latest=...): ...
def storage_uri_for_key(key): ...
