# Stubs for _curses (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

ALL_MOUSE_EVENTS = ... # type: int
A_ALTCHARSET = ... # type: int
A_ATTRIBUTES = ... # type: int
A_BLINK = ... # type: int
A_BOLD = ... # type: int
A_CHARTEXT = ... # type: int
A_COLOR = ... # type: int
A_DIM = ... # type: int
A_HORIZONTAL = ... # type: int
A_INVIS = ... # type: int
A_LEFT = ... # type: int
A_LOW = ... # type: int
A_NORMAL = ... # type: int
A_PROTECT = ... # type: int
A_REVERSE = ... # type: int
A_RIGHT = ... # type: int
A_STANDOUT = ... # type: int
A_TOP = ... # type: int
A_UNDERLINE = ... # type: int
A_VERTICAL = ... # type: int
BUTTON1_CLICKED = ... # type: int
BUTTON1_DOUBLE_CLICKED = ... # type: int
BUTTON1_PRESSED = ... # type: int
BUTTON1_RELEASED = ... # type: int
BUTTON1_TRIPLE_CLICKED = ... # type: int
BUTTON2_CLICKED = ... # type: int
BUTTON2_DOUBLE_CLICKED = ... # type: int
BUTTON2_PRESSED = ... # type: int
BUTTON2_RELEASED = ... # type: int
BUTTON2_TRIPLE_CLICKED = ... # type: int
BUTTON3_CLICKED = ... # type: int
BUTTON3_DOUBLE_CLICKED = ... # type: int
BUTTON3_PRESSED = ... # type: int
BUTTON3_RELEASED = ... # type: int
BUTTON3_TRIPLE_CLICKED = ... # type: int
BUTTON4_CLICKED = ... # type: int
BUTTON4_DOUBLE_CLICKED = ... # type: int
BUTTON4_PRESSED = ... # type: int
BUTTON4_RELEASED = ... # type: int
BUTTON4_TRIPLE_CLICKED = ... # type: int
BUTTON_ALT = ... # type: int
BUTTON_CTRL = ... # type: int
BUTTON_SHIFT = ... # type: int
COLOR_BLACK = ... # type: int
COLOR_BLUE = ... # type: int
COLOR_CYAN = ... # type: int
COLOR_GREEN = ... # type: int
COLOR_MAGENTA = ... # type: int
COLOR_RED = ... # type: int
COLOR_WHITE = ... # type: int
COLOR_YELLOW = ... # type: int
ERR = ... # type: int
KEY_A1 = ... # type: int
KEY_A3 = ... # type: int
KEY_B2 = ... # type: int
KEY_BACKSPACE = ... # type: int
KEY_BEG = ... # type: int
KEY_BREAK = ... # type: int
KEY_BTAB = ... # type: int
KEY_C1 = ... # type: int
KEY_C3 = ... # type: int
KEY_CANCEL = ... # type: int
KEY_CATAB = ... # type: int
KEY_CLEAR = ... # type: int
KEY_CLOSE = ... # type: int
KEY_COMMAND = ... # type: int
KEY_COPY = ... # type: int
KEY_CREATE = ... # type: int
KEY_CTAB = ... # type: int
KEY_DC = ... # type: int
KEY_DL = ... # type: int
KEY_DOWN = ... # type: int
KEY_EIC = ... # type: int
KEY_END = ... # type: int
KEY_ENTER = ... # type: int
KEY_EOL = ... # type: int
KEY_EOS = ... # type: int
KEY_EXIT = ... # type: int
KEY_F0 = ... # type: int
KEY_F1 = ... # type: int
KEY_F10 = ... # type: int
KEY_F11 = ... # type: int
KEY_F12 = ... # type: int
KEY_F13 = ... # type: int
KEY_F14 = ... # type: int
KEY_F15 = ... # type: int
KEY_F16 = ... # type: int
KEY_F17 = ... # type: int
KEY_F18 = ... # type: int
KEY_F19 = ... # type: int
KEY_F2 = ... # type: int
KEY_F20 = ... # type: int
KEY_F21 = ... # type: int
KEY_F22 = ... # type: int
KEY_F23 = ... # type: int
KEY_F24 = ... # type: int
KEY_F25 = ... # type: int
KEY_F26 = ... # type: int
KEY_F27 = ... # type: int
KEY_F28 = ... # type: int
KEY_F29 = ... # type: int
KEY_F3 = ... # type: int
KEY_F30 = ... # type: int
KEY_F31 = ... # type: int
KEY_F32 = ... # type: int
KEY_F33 = ... # type: int
KEY_F34 = ... # type: int
KEY_F35 = ... # type: int
KEY_F36 = ... # type: int
KEY_F37 = ... # type: int
KEY_F38 = ... # type: int
KEY_F39 = ... # type: int
KEY_F4 = ... # type: int
KEY_F40 = ... # type: int
KEY_F41 = ... # type: int
KEY_F42 = ... # type: int
KEY_F43 = ... # type: int
KEY_F44 = ... # type: int
KEY_F45 = ... # type: int
KEY_F46 = ... # type: int
KEY_F47 = ... # type: int
KEY_F48 = ... # type: int
KEY_F49 = ... # type: int
KEY_F5 = ... # type: int
KEY_F50 = ... # type: int
KEY_F51 = ... # type: int
KEY_F52 = ... # type: int
KEY_F53 = ... # type: int
KEY_F54 = ... # type: int
KEY_F55 = ... # type: int
KEY_F56 = ... # type: int
KEY_F57 = ... # type: int
KEY_F58 = ... # type: int
KEY_F59 = ... # type: int
KEY_F6 = ... # type: int
KEY_F60 = ... # type: int
KEY_F61 = ... # type: int
KEY_F62 = ... # type: int
KEY_F63 = ... # type: int
KEY_F7 = ... # type: int
KEY_F8 = ... # type: int
KEY_F9 = ... # type: int
KEY_FIND = ... # type: int
KEY_HELP = ... # type: int
KEY_HOME = ... # type: int
KEY_IC = ... # type: int
KEY_IL = ... # type: int
KEY_LEFT = ... # type: int
KEY_LL = ... # type: int
KEY_MARK = ... # type: int
KEY_MAX = ... # type: int
KEY_MESSAGE = ... # type: int
KEY_MIN = ... # type: int
KEY_MOUSE = ... # type: int
KEY_MOVE = ... # type: int
KEY_NEXT = ... # type: int
KEY_NPAGE = ... # type: int
KEY_OPEN = ... # type: int
KEY_OPTIONS = ... # type: int
KEY_PPAGE = ... # type: int
KEY_PREVIOUS = ... # type: int
KEY_PRINT = ... # type: int
KEY_REDO = ... # type: int
KEY_REFERENCE = ... # type: int
KEY_REFRESH = ... # type: int
KEY_REPLACE = ... # type: int
KEY_RESET = ... # type: int
KEY_RESIZE = ... # type: int
KEY_RESTART = ... # type: int
KEY_RESUME = ... # type: int
KEY_RIGHT = ... # type: int
KEY_SAVE = ... # type: int
KEY_SBEG = ... # type: int
KEY_SCANCEL = ... # type: int
KEY_SCOMMAND = ... # type: int
KEY_SCOPY = ... # type: int
KEY_SCREATE = ... # type: int
KEY_SDC = ... # type: int
KEY_SDL = ... # type: int
KEY_SELECT = ... # type: int
KEY_SEND = ... # type: int
KEY_SEOL = ... # type: int
KEY_SEXIT = ... # type: int
KEY_SF = ... # type: int
KEY_SFIND = ... # type: int
KEY_SHELP = ... # type: int
KEY_SHOME = ... # type: int
KEY_SIC = ... # type: int
KEY_SLEFT = ... # type: int
KEY_SMESSAGE = ... # type: int
KEY_SMOVE = ... # type: int
KEY_SNEXT = ... # type: int
KEY_SOPTIONS = ... # type: int
KEY_SPREVIOUS = ... # type: int
KEY_SPRINT = ... # type: int
KEY_SR = ... # type: int
KEY_SREDO = ... # type: int
KEY_SREPLACE = ... # type: int
KEY_SRESET = ... # type: int
KEY_SRIGHT = ... # type: int
KEY_SRSUME = ... # type: int
KEY_SSAVE = ... # type: int
KEY_SSUSPEND = ... # type: int
KEY_STAB = ... # type: int
KEY_SUNDO = ... # type: int
KEY_SUSPEND = ... # type: int
KEY_UNDO = ... # type: int
KEY_UP = ... # type: int
OK = ... # type: int
REPORT_MOUSE_POSITION = ... # type: int
_C_API = ... # type: Any
version = ... # type: bytes

def baudrate(*args, **kwargs): ...
def beep(*args, **kwargs): ...
def can_change_color(*args, **kwargs): ...
def cbreak(*args, **kwargs): ...
def color_content(*args, **kwargs): ...
def color_pair(*args, **kwargs): ...
def curs_set(*args, **kwargs): ...
def def_prog_mode(*args, **kwargs): ...
def def_shell_mode(*args, **kwargs): ...
def delay_output(*args, **kwargs): ...
def doupdate(*args, **kwargs): ...
def echo(*args, **kwargs): ...
def endwin(*args, **kwargs): ...
def erasechar(*args, **kwargs): ...
def filter(*args, **kwargs): ...
def flash(*args, **kwargs): ...
def flushinp(*args, **kwargs): ...
def getmouse(*args, **kwargs): ...
def getsyx(*args, **kwargs): ...
def getwin(*args, **kwargs): ...
def halfdelay(*args, **kwargs): ...
def has_colors(*args, **kwargs): ...
def has_ic(*args, **kwargs): ...
def has_il(*args, **kwargs): ...
def has_key(*args, **kwargs): ...
def init_color(*args, **kwargs): ...
def init_pair(*args, **kwargs): ...
def initscr(*args, **kwargs): ...
def intrflush(*args, **kwargs): ...
def is_term_resized(*args, **kwargs): ...
def isendwin(*args, **kwargs): ...
def keyname(*args, **kwargs): ...
def killchar(*args, **kwargs): ...
def longname(*args, **kwargs): ...
def meta(*args, **kwargs): ...
def mouseinterval(*args, **kwargs): ...
def mousemask(*args, **kwargs): ...
def napms(*args, **kwargs): ...
def newpad(*args, **kwargs): ...
def newwin(*args, **kwargs): ...
def nl(*args, **kwargs): ...
def nocbreak(*args, **kwargs): ...
def noecho(*args, **kwargs): ...
def nonl(*args, **kwargs): ...
def noqiflush(*args, **kwargs): ...
def noraw(*args, **kwargs): ...
def pair_content(*args, **kwargs): ...
def pair_number(*args, **kwargs): ...
def putp(*args, **kwargs): ...
def qiflush(*args, **kwargs): ...
def raw(*args, **kwargs): ...
def reset_prog_mode(*args, **kwargs): ...
def reset_shell_mode(*args, **kwargs): ...
def resetty(*args, **kwargs): ...
def resize_term(*args, **kwargs): ...
def resizeterm(*args, **kwargs): ...
def savetty(*args, **kwargs): ...
def setsyx(*args, **kwargs): ...
def setupterm(*args, **kwargs): ...
def start_color(*args, **kwargs): ...
def termattrs(*args, **kwargs): ...
def termname(*args, **kwargs): ...
def tigetflag(*args, **kwargs): ...
def tigetnum(*args, **kwargs): ...
def tigetstr(*args, **kwargs): ...
def tparm(*args, **kwargs): ...
def typeahead(*args, **kwargs): ...
def unctrl(*args, **kwargs): ...
def unget_wch(*args, **kwargs): ...
def ungetch(*args, **kwargs): ...
def ungetmouse(*args, **kwargs): ...
def update_lines_cols(*args, **kwargs): ...
def use_default_colors(*args, **kwargs): ...
def use_env(*args, **kwargs): ...

class error(Exception): ...
