# Stubs for email.mime.text (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from email.mime.nonmultipart import MIMENonMultipart

class MIMEText(MIMENonMultipart):
    def __init__(self, _text, _subtype=..., _charset=...) -> None: ...
