# Stubs for requests.packages.urllib3.packages.ssl_match_hostname._implementation (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

class CertificateError(ValueError): ...

def match_hostname(cert, hostname): ...
