# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib_robotparser (Python 2.7)

from six.moves.urllib.robotparser import (
    RobotFileParser as RobotFileParser,
)
