# Stubs for requests.packages.urllib3.util.connection (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

poll = ...  # type: Any
select = ...  # type: Any

def is_connection_dropped(conn): ...
def create_connection(address, timeout=..., source_address=..., socket_options=...): ...
