# Stubs for doctest

# NOTE: These are incomplete!

from typing import Any, Tuple

# TODO arguments missing
def testmod(m: Any = ..., name: str = ..., globs: Any = ...,
            verbose: bool = ...) -> Tuple[int, int]: ...
