from typing import Any

class addbase:
    fp = ... # type: Any
    read = ... # type: Any
    readline = ... # type: Any
    readlines = ... # type: Any
    fileno = ... # type: Any
    __iter__ = ... # type: Any
    next = ... # type: Any
    def __init__(self, fp) -> None: ...
    def close(self): ...

class addclosehook(addbase):
    closehook = ... # type: Any
    hookargs = ... # type: Any
    def __init__(self, fp, closehook, *hookargs) -> None: ...
    def close(self): ...

class addinfo(addbase):
    headers = ... # type: Any
    def __init__(self, fp, headers) -> None: ...
    def info(self): ...

class addinfourl(addbase):
    headers = ... # type: Any
    url = ... # type: Any
    code = ... # type: Any
    def __init__(self, fp, headers, url, code=...) -> None: ...
    def info(self): ...
    def getcode(self): ...
    def geturl(self): ...
