# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib.robotparser (Python 3.2)

from urllib.robotparser import RobotFileParser as RobotFileParser
