# Stubs for requests.packages.urllib3.packages.ssl_match_hostname (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

import ssl

CertificateError = ssl.CertificateError
match_hostname = ssl.match_hostname
