# Stubs for sqlalchemy.engine.url (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from .. import dialects

## registry = dialects.registry

class URL:
    drivername = ... # type: Any
    username = ... # type: Any
    password = ... # type: Any
    host = ... # type: Any
    port = ... # type: Any
    database = ... # type: Any
    query = ... # type: Any
    def __init__(self, drivername, username=..., password=..., host=..., port=..., database=..., query=...) -> None: ...
    def __to_string__(self, hide_password=...): ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    def get_backend_name(self): ...
    def get_driver_name(self): ...
    def get_dialect(self): ...
    def translate_connect_args(self, names=..., **kw): ...

def make_url(name_or_url): ...
