# Stubs for sqlalchemy.dialects.mysql (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from . import base

BIGINT = base.BIGINT
BINARY = base.BINARY
BIT = base.BIT
BLOB = base.BLOB
BOOLEAN = base.BOOLEAN
CHAR = base.CHAR
DATE = base.DATE
DATETIME = base.DATETIME
DECIMAL = base.DECIMAL
DOUBLE = base.DOUBLE
ENUM = base.ENUM
FLOAT = base.FLOAT
INTEGER = base.INTEGER
LONGBLOB = base.LONGBLOB
LONGTEXT = base.LONGTEXT
MEDIUMBLOB = base.MEDIUMBLOB
MEDIUMINT = base.MEDIUMINT
MEDIUMTEXT = base.MEDIUMTEXT
NCHAR = base.NCHAR
NVARCHAR = base.NVARCHAR
NUMERIC = base.NUMERIC
SET = base.SET
SMALLINT = base.SMALLINT
REAL = base.REAL
TEXT = base.TEXT
TIME = base.TIME
TIMESTAMP = base.TIMESTAMP
TINYBLOB = base.TINYBLOB
TINYINT = base.TINYINT
TINYTEXT = base.TINYTEXT
VARBINARY = base.VARBINARY
VARCHAR = base.VARCHAR
YEAR = base.YEAR
## dialect = base.dialect
