"""Stub file for the 'xxsubtype' module."""

from typing import Any

def bench(obj: Any, name: str, n:int = ...) -> float: ...

class spamdict(dict):
    state = ...  # type: int
    def getstate(self) -> int: ...
    def setstate(self, a: int) -> None: ...

class spamlist(list):
    state = ...  # type: int
    def getstate(self) -> int: ...
    def setstate(self, a: int) -> None: ...
    def classmeth(self, *args, **kwargs) -> tuple: ...
    def staticmeth(self, *args, **kwargs) -> tuple: ...
