# Stubs for _io (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class _IOBase:
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def closed(self): ...
    def close(self): ...
    def fileno(self): ...
    def flush(self): ...
    def isatty(self): ...
    def readable(self): ...
    def readline(self, size: int = ...): ...
    def readlines(self, hint: int = ...): ...
    def seek(self, offset, whence=...): ...
    def seekable(self): ...
    def tell(self): ...
    def truncate(self, size: int = ...) -> int: ...
    def writable(self): ...
    def writelines(self, lines): ...
    def __del__(self): ...
    def __enter__(self): ...
    def __exit__(self, exc_type, exc_val, exc_tb): ...
    def __iter__(self): ...
    def __next__(self): ...

class _BufferedIOBase(_IOBase):
    def detach(self): ...
    def read(self, size: int = ...): ...
    def read1(self, size: int = ...): ...
    def readinto(self, b): ...
    def write(self, b): ...

class _RawIOBase(_IOBase):
    def read(self, size: int = ...): ...
    def readall(self): ...

class _TextIOBase(_IOBase):
    encoding = ...  # type: Any
    errors = ...  # type: Any
    newlines = ...  # type: Any
    def detach(self): ...
    def read(self, size: int = ...): ...
    def readline(self, size: int = ...): ...
    def write(self, b): ...
