# Stubs for _random

# NOTE: These are incomplete!

from typing import Any

class Random:
    def seed(self, x: Any = ...) -> None: ...
    def getstate(self) -> tuple: ...
    def setstate(self, state: tuple) -> None: ...
    def random(self) -> float: ...
    def getrandbits(self, k: int) -> int: ...
