# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib_request (Python 3.2)

from six.moves.urllib.request import (
    install_opener as install_opener,
    build_opener as build_opener,
    BaseHandler as BaseHandler,
    OpenerDirector as OpenerDirector,
    HTTPRedirectHandler as HTTPRedirectHandler,

    # urlopen as urlopen,
    # pathname2url as pathname2url,
    # url2pathname as url2pathname,
    # getproxies as getproxies,
    # Request as Request,
    # HTTPDefaultErrorHandler as HTTPDefaultErrorHandler,
    # HTTPCookieProcessor as HTTPCookieProcessor,
    # ProxyHandler as ProxyHandler,
    # HTTPPasswordMgr as HTTPPasswordMgr,
    # HTTPPasswordMgrWithDefaultRealm as HTTPPasswordMgrWithDefaultRealm,
    # AbstractBasicAuthHandler as AbstractBasicAuthHandler,
    # HTTPBasicAuthHandler as HTTPBasicAuthHandler,
    # ProxyBasicAuthHandler as ProxyBasicAuthHandler,
    # AbstractDigestAuthHandler as AbstractDigestAuthHandler,
    # HTTPDigestAuthHandler as HTTPDigestAuthHandler,
    # ProxyDigestAuthHandler as ProxyDigestAuthHandler,
    # HTTPHandler as HTTPHandler,
    # HTTPSHandler as HTTPSHandler,
    # FileHandler as FileHandler,
    # FTPHandler as FTPHandler,
    # CacheFTPHandler as CacheFTPHandler,
    # UnknownHandler as UnknownHandler,
    # HTTPErrorProcessor as HTTPErrorProcessor,
    # urlretrieve as urlretrieve,
    # urlcleanup as urlcleanup,
    # URLopener as URLopener,
    # FancyURLopener as FancyURLopener,
    # proxy_bypass as proxy_bypass,
)
