# Stubs for linecache (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def getline(filename, lineno, module_globals=None): ...
def clearcache(): ...
def checkcache(filename=None): ...
