# Stubs for email.base64mime (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def header_length(bytearray): ...
def header_encode(header_bytes, charset=...): ...
def body_encode(s, maxlinelen=..., eol=...): ...
def decode(string): ...

body_decode = ...  # type: Any
decodestring = ...  # type: Any
