# Stubs for pprint (Python 2)
#
# NOTE: Based on a dynamically typed automatically generated by stubgen.

from typing import IO, Any

def pprint(object: Any, stream: IO[str] = ..., indent: int = ..., width: int = ...,
           depth: int = ...) -> None: ...
def pformat(object, indent=..., width=..., depth=...): ...
def saferepr(object): ...
def isreadable(object): ...
def isrecursive(object): ...

class PrettyPrinter:
    def __init__(self, indent: int = ..., width: int = ..., depth: int = ...,
                 stream: IO[str] = ...) -> None: ...
    def pprint(self, object): ...
    def pformat(self, object): ...
    def isrecursive(self, object): ...
    def isreadable(self, object): ...
    def format(self, object, context, maxlevels, level): ...
