# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib_error (Python 3.2)

from six.moves.urllib.error import (
    URLError as URLError,
    HTTPError as HTTPError,
    ContentTooShortError as ContentTooShortError,
)
