# Stubs for email._policybase (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class _PolicyBase:
    def __init__(self, **kw) -> None: ...
    def clone(self, **kw): ...
    def __setattr__(self, name, value): ...
    def __add__(self, other): ...

class Policy(_PolicyBase):
    raise_on_defect = ...  # type: Any
    linesep = ...  # type: Any
    cte_type = ...  # type: Any
    max_line_length = ...  # type: Any
    def handle_defect(self, obj, defect): ...
    def register_defect(self, obj, defect): ...
    def header_max_count(self, name): ...
    def header_source_parse(self, sourcelines): ...
    def header_store_parse(self, name, value): ...
    def header_fetch_parse(self, name, value): ...
    def fold(self, name, value): ...
    def fold_binary(self, name, value): ...

class Compat32(Policy):
    def header_source_parse(self, sourcelines): ...
    def header_store_parse(self, name, value): ...
    def header_fetch_parse(self, name, value): ...
    def fold(self, name, value): ...
    def fold_binary(self, name, value): ...

compat32 = ...  # type: Any
