# Stubs for email.iterators (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def walk(self): ...
def body_line_iterator(msg, decode=...): ...
def typed_subpart_iterator(msg, maintype=..., subtype=...): ...
