# Stubs for sqlalchemy.engine (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def create_engine(*args, **kwargs): ...
def engine_from_config(configuration, prefix=..., **kwargs): ...
