# Stubs for locale (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from _locale import *

class _unicode: ...

CHAR_MAX = ... # type: Any
LC_ALL = ... # type: Any
LC_COLLATE = ... # type: Any
LC_CTYPE = ... # type: Any
LC_MESSAGES = ... # type: Any
LC_MONETARY = ... # type: Any
LC_NUMERIC = ... # type: Any
LC_TIME = ... # type: Any
Error = ... # type: Any

def localeconv(): ...
def strcoll(a, b): ...
def strxfrm(s): ...
def format(percent, value, grouping=False, monetary=False, *additional): ...
def format_string(f, val, grouping=False): ...
def currency(val, symbol=True, grouping=False, international=False): ...
def str(val): ...
def atof(string, func=...): ...
def atoi(str): ...
def normalize(localename): ...
def getdefaultlocale(envvars=...): ...
def getlocale(category=...): ...
def resetlocale(category=...): ...
def getpreferredencoding(do_setlocale=True): ...
