# Stubs for quopri (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def encode(input, output, quotetabs, header=0): ...
def encodestring(s, quotetabs=0, header=0): ...
def decode(input, output, header=0): ...
def decodestring(s, header=0): ...
