# Stubs for email.charset (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def add_charset(charset, header_enc=..., body_enc=..., output_charset=...): ...
def add_alias(alias, canonical): ...
def add_codec(charset, codecname): ...

class Charset:
    input_charset = ...  # type: Any
    header_encoding = ...  # type: Any
    body_encoding = ...  # type: Any
    output_charset = ...  # type: Any
    input_codec = ...  # type: Any
    output_codec = ...  # type: Any
    def __init__(self, input_charset=...) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def get_body_encoding(self): ...
    def get_output_charset(self): ...
    def header_encode(self, string): ...
    def header_encode_lines(self, string, maxlengths): ...
    def body_encode(self, string): ...
