# Stubs for compileall (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def compile_dir(dir, maxlevels=..., ddir=..., force=..., rx=..., quiet=...): ...
def compile_file(fullname, ddir=..., force=..., rx=..., quiet=...): ...
def compile_path(skip_curdir=..., maxlevels=..., force=..., quiet=...): ...
