def bisect(a: list, x: object, lo: int = ..., hi: int = ...) -> int: ...
def bisect_left(a: list, x: object, lo: int = ..., hi: int = ...) -> int: ...
def bisect_right(a: list, x: object, lo: int = ..., hi: int = ...) -> int: ...
def insort_left(a: list, x: object, lo: int = ..., hi: int = ...) -> int: ...
def insort_right(a: list, x: object, lo: int = ..., hi: int = ...) -> int: ...
def insort(a: list, x: object, lo: int = ..., hi: int = ...) -> int: ...
