# Stubs for sqlalchemy.dialects (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Names in __all__ with no definition:
#   firebird
#   mssql
#   mysql
#   oracle
#   postgresql
#   sqlite
#   sybase
