# Stubs for google.protobuf.reflection (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

class GeneratedProtocolMessageType(type):
    def __new__(cls, name, bases, dictionary): ...
    def __init__(cls, name, bases, dictionary) -> None: ...

def ParseMessage(descriptor, byte_str): ...
def MakeClass(descriptor): ...
