# Stubs for multiprocessing

from typing import Any

class Lock(): ...
class Process(): ...

class Queue():
    def get(block: bool = ..., timeout: float = ...) -> Any: ...

class Value():
    def __init__(typecode_or_type: str, *args: Any, lock: bool = ...) -> None: ...

# ----- multiprocessing function stubs -----
def cpu_count() -> int: ...
