# Stubs for boto.ec2 (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

RegionData = ... # type: Any

def regions(**kw_params): ...
def connect_to_region(region_name, **kw_params): ...
def get_region(region_name, **kw_params): ...
