# Stubs for sqlalchemy.util.deprecations (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from . import langhelpers

decorator = langhelpers.decorator

def warn_deprecated(msg, stacklevel=...): ...
def warn_pending_deprecation(msg, stacklevel=...): ...
def deprecated(version, message=..., add_deprecation_to_docstring=...): ...
def pending_deprecation(version, message=..., add_deprecation_to_docstring=...): ...
def inject_docstring_text(doctext, injecttext, pos): ...
