# Stubs for html.entities (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

name2codepoint = ... # type: Any
html5 = ... # type: Any
codepoint2name = ... # type: Any
entitydefs = ... # type: Any
