# Stubs for unicodedata (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

ucd_3_2_0 = ... # type: Any
ucnhash_CAPI = ... # type: Any
unidata_version = ... # type: str

def bidirectional(unichr): ...
def category(unichr): ...
def combining(unichr): ...
def decimal(chr, default=...): ...
def decomposition(unichr): ...
def digit(chr, default=...): ...
def east_asian_width(unichr): ...
def lookup(name): ...
def mirrored(unichr): ...
def name(chr, default=...): ...
def normalize(form, unistr): ...
def numeric(chr, default=...): ...

class UCD:
    unidata_version = ... # type: Any
    def bidirectional(self, unichr): ...
    def category(self, unichr): ...
    def combining(self, unichr): ...
    def decimal(self, chr, default=...): ...
    def decomposition(self, unichr): ...
    def digit(self, chr, default=...): ...
    def east_asian_width(self, unichr): ...
    def lookup(self, name): ...
    def mirrored(self, unichr): ...
    def name(self, chr, default=...): ...
    def normalize(self, form, unistr): ...
    def numeric(self, chr, default=...): ...
