# Stubs for requests.structures (Python 3)

from typing import Any
import collections

class CaseInsensitiveDict(collections.MutableMapping):
    def __init__(self, data=..., **kwargs) -> None: ...
    def __setitem__(self, key, value): ...
    def __getitem__(self, key): ...
    def __delitem__(self, key): ...
    def __iter__(self): ...
    def __len__(self): ...
    def lower_items(self): ...
    def __eq__(self, other): ...
    def copy(self): ...

class LookupDict(dict):
    name = ...  # type: Any
    def __init__(self, name=...) -> None: ...
    def __getitem__(self, key): ...
    def get(self, key, default=...): ...
