# Stubs for thrift.protocol (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Names in __all__ with no definition:
#   TBase
#   TBinaryProtocol
#   TCompactProtocol
#   TJSONProtocol
#   TProtocol
#   fastbinary
