# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib.error (Python 3.2)

from urllib.error import URLError as URLError
from urllib.error import HTTPError as HTTPError
from urllib.error import ContentTooShortError as ContentTooShortError
