# Stubs for requests.status_codes (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from .structures import LookupDict

codes = ...  # type: Any
