# Stubs for email.mime.multipart (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from email.mime.base import MIMEBase

class MIMEMultipart(MIMEBase):
    def __init__(self, _subtype=..., boundary=..., _subparts=..., **_params) -> None: ...
