# Stubs for curses (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from _curses import *
# Stubgen imports a python version of has_key only if it's not present
#in _curses (which it is in this stub)
# from .has_key import has_key as has_key

def initscr(): ...
def start_color(): ...
def wrapper(func, *args, **kwds): ...
