# Stubs for email.encoders (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def encode_base64(msg): ...
def encode_quopri(msg): ...
def encode_7or8bit(msg): ...
def encode_noop(msg): ...
