# Stubs for email.mime.image (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from email.mime.nonmultipart import MIMENonMultipart

class MIMEImage(MIMENonMultipart):
    def __init__(self, _imagedata, _subtype=..., _encoder=..., **_params) -> None: ...
