# Stubs for email.mime.base (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# import message

# TODO
#class MIMEBase(message.Message):
class MIMEBase:
    def __init__(self, _maintype, _subtype, **_params) -> None: ...
