# Stubs for mimetypes (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class MimeTypes:
    encodings_map = ... # type: Any
    suffix_map = ... # type: Any
    types_map = ... # type: Any
    types_map_inv = ... # type: Any
    def __init__(self, filenames=..., strict=True): ...
    def add_type(self, type, ext, strict=True): ...
    def guess_type(self, url, strict=True): ...
    def guess_all_extensions(self, type, strict=True): ...
    def guess_extension(self, type, strict=True): ...
    def read(self, filename, strict=True): ...
    def readfp(self, fp, strict=True): ...
    def read_windows_registry(self, strict=True): ...

def guess_type(url, strict=True): ...
def guess_all_extensions(type, strict=True): ...
def guess_extension(type, strict=True): ...
def add_type(type, ext, strict=True): ...
def init(files=None): ...
def read_mime_types(file): ...
