# Stubs for wsgiref.validate (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class WSGIWarning(Warning): ...

def validator(application): ...

class InputWrapper:
    input = ... # type: Any
    def __init__(self, wsgi_input): ...
    def read(self, *args): ...
    def readline(self): ...
    def readlines(self, *args): ...
    def __iter__(self): ...
    def close(self): ...

class ErrorWrapper:
    errors = ... # type: Any
    def __init__(self, wsgi_errors): ...
    def write(self, s): ...
    def flush(self): ...
    def writelines(self, seq): ...
    def close(self): ...

class WriteWrapper:
    writer = ... # type: Any
    def __init__(self, wsgi_writer): ...
    def __call__(self, s): ...

class PartialIteratorWrapper:
    iterator = ... # type: Any
    def __init__(self, wsgi_iterator): ...
    def __iter__(self): ...

class IteratorWrapper:
    original_iterator = ... # type: Any
    iterator = ... # type: Any
    closed = ... # type: Any
    check_start_response = ... # type: Any
    def __init__(self, wsgi_iterator, check_start_response): ...
    def __iter__(self): ...
    def next(self): ...
    def close(self): ...
    def __del__(self): ...
