# Stubs for email.mime.message (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from email.mime.nonmultipart import MIMENonMultipart

class MIMEMessage(MIMENonMultipart):
    def __init__(self, _msg, _subtype=...) -> None: ...
