# Stubs for email.mime.base (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from email import message

class MIMEBase(message.Message):
    def __init__(self, _maintype, _subtype, **_params) -> None: ...
