# Stubs for email.parser (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
import email.feedparser

FeedParser = email.feedparser.FeedParser
BytesFeedParser = email.feedparser.BytesFeedParser

class Parser:
    policy = ...  # type: Any
    def __init__(self, _class=..., *, policy=...) -> None: ...
    def parse(self, fp, headersonly=...): ...
    def parsestr(self, text, headersonly=...): ...

class HeaderParser(Parser):
    def parse(self, fp, headersonly=...): ...
    def parsestr(self, text, headersonly=...): ...

class BytesParser:
    parser = ...  # type: Any
    def __init__(self, *args, **kw) -> None: ...
    def parse(self, fp, headersonly=...): ...
    def parsebytes(self, text, headersonly=...): ...

class BytesHeaderParser(BytesParser):
    def parse(self, fp, headersonly=...): ...
    def parsebytes(self, text, headersonly=...): ...
