# Stubs for requests.packages (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

class VendorAlias:
    def __init__(self, package_names) -> None: ...
    def find_module(self, fullname, path=...): ...
    def load_module(self, name): ...
