# Stubs for redis.utils (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

HIREDIS_AVAILABLE = ... # type: Any

def from_url(url, db=..., **kwargs): ...
def pipeline(redis_obj): ...

class dummy: ...
