# Stubs for email.mime.nonmultipart (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from email.mime.base import MIMEBase

class MIMENonMultipart(MIMEBase):
    def attach(self, payload): ...
