import logging
from typing import Any

rootlogger = ... # type: Any

class Identified(object):
    def _should_log_debug(self) -> bool: ...
    def _should_log_info(self) -> bool: ...

class InstanceLogger(object): ...

def instance_logger(instance, echoflag) -> None: ...

class echo_property(object): ...
