# Stubs for Crypto.Cipher (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Names in __all__ with no definition:
#   AES
#   ARC2
#   ARC4
#   Blowfish
#   CAST
#   DES
#   DES3
#   PKCS1_OAEP
#   PKCS1_v1_5
#   XOR
