# Stubs for sqlalchemy.util (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from . import compat
from . import _collections
from . import langhelpers
from . import deprecations

callable = compat.callable
cmp = compat.cmp
reduce = compat.reduce
threading = compat.threading
py3k = compat.py3k
py33 = compat.py33
py2k = compat.py2k
jython = compat.jython
pypy = compat.pypy
cpython = compat.cpython
win32 = compat.win32
pickle = compat.pickle
dottedgetter = compat.dottedgetter
parse_qsl = compat.parse_qsl
namedtuple = compat.namedtuple
next = compat.next
reraise = compat.reraise
raise_from_cause = compat.raise_from_cause
text_type = compat.text_type
safe_kwarg = compat.safe_kwarg
string_types = compat.string_types
int_types = compat.int_types
binary_type = compat.binary_type
nested = compat.nested
quote_plus = compat.quote_plus
with_metaclass = compat.with_metaclass
print_ = compat.print_
itertools_filterfalse = compat.itertools_filterfalse
u = compat.u
ue = compat.ue
b = compat.b
unquote_plus = compat.unquote_plus
unquote = compat.unquote
b64decode = compat.b64decode
b64encode = compat.b64encode
byte_buffer = compat.byte_buffer
itertools_filter = compat.itertools_filter
iterbytes = compat.iterbytes
StringIO = compat.StringIO
inspect_getargspec = compat.inspect_getargspec
zip_longest = compat.zip_longest
KeyedTuple = _collections.KeyedTuple
ImmutableContainer = _collections.ImmutableContainer
immutabledict = _collections.immutabledict
Properties = _collections.Properties
OrderedProperties = _collections.OrderedProperties
ImmutableProperties = _collections.ImmutableProperties
OrderedDict = _collections.OrderedDict
OrderedSet = _collections.OrderedSet
IdentitySet = _collections.IdentitySet
OrderedIdentitySet = _collections.OrderedIdentitySet
column_set = _collections.column_set
column_dict = _collections.column_dict
ordered_column_set = _collections.ordered_column_set
populate_column_dict = _collections.populate_column_dict
unique_list = _collections.unique_list
UniqueAppender = _collections.UniqueAppender
PopulateDict = _collections.PopulateDict
EMPTY_SET = _collections.EMPTY_SET
to_list = _collections.to_list
to_set = _collections.to_set
to_column_set = _collections.to_column_set
update_copy = _collections.update_copy
flatten_iterator = _collections.flatten_iterator
has_intersection = _collections.has_intersection
LRUCache = _collections.LRUCache
ScopedRegistry = _collections.ScopedRegistry
ThreadLocalRegistry = _collections.ThreadLocalRegistry
WeakSequence = _collections.WeakSequence
coerce_generator_arg = _collections.coerce_generator_arg
lightweight_named_tuple = _collections.lightweight_named_tuple
iterate_attributes = langhelpers.iterate_attributes
class_hierarchy = langhelpers.class_hierarchy
portable_instancemethod = langhelpers.portable_instancemethod
unbound_method_to_callable = langhelpers.unbound_method_to_callable
getargspec_init = langhelpers.getargspec_init
format_argspec_init = langhelpers.format_argspec_init
format_argspec_plus = langhelpers.format_argspec_plus
get_func_kwargs = langhelpers.get_func_kwargs
get_cls_kwargs = langhelpers.get_cls_kwargs
decorator = langhelpers.decorator
as_interface = langhelpers.as_interface
memoized_property = langhelpers.memoized_property
memoized_instancemethod = langhelpers.memoized_instancemethod
md5_hex = langhelpers.md5_hex
group_expirable_memoized_property = langhelpers.group_expirable_memoized_property
dependencies = langhelpers.dependencies
decode_slice = langhelpers.decode_slice
monkeypatch_proxied_specials = langhelpers.monkeypatch_proxied_specials
asbool = langhelpers.asbool
bool_or_str = langhelpers.bool_or_str
coerce_kw_type = langhelpers.coerce_kw_type
duck_type_collection = langhelpers.duck_type_collection
assert_arg_type = langhelpers.assert_arg_type
symbol = langhelpers.symbol
dictlike_iteritems = langhelpers.dictlike_iteritems
classproperty = langhelpers.classproperty
set_creation_order = langhelpers.set_creation_order
warn_exception = langhelpers.warn_exception
warn = langhelpers.warn
NoneType = langhelpers.NoneType
constructor_copy = langhelpers.constructor_copy
methods_equivalent = langhelpers.methods_equivalent
chop_traceback = langhelpers.chop_traceback
asint = langhelpers.asint
generic_repr = langhelpers.generic_repr
counter = langhelpers.counter
PluginLoader = langhelpers.PluginLoader
hybridproperty = langhelpers.hybridproperty
hybridmethod = langhelpers.hybridmethod
safe_reraise = langhelpers.safe_reraise
get_callable_argspec = langhelpers.get_callable_argspec
only_once = langhelpers.only_once
attrsetter = langhelpers.attrsetter
ellipses_string = langhelpers.ellipses_string
warn_limited = langhelpers.warn_limited
map_bits = langhelpers.map_bits
MemoizedSlots = langhelpers.MemoizedSlots
EnsureKWArgType = langhelpers.EnsureKWArgType
warn_deprecated = deprecations.warn_deprecated
warn_pending_deprecation = deprecations.warn_pending_deprecation
deprecated = deprecations.deprecated
pending_deprecation = deprecations.pending_deprecation
inject_docstring_text = deprecations.inject_docstring_text
