# Stubs for email.utils (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

import email._parseaddr

mktime_tz = email._parseaddr.mktime_tz
parsedate = email._parseaddr.parsedate
parsedate_tz = email._parseaddr.parsedate_tz

def formataddr(pair, charset=...): ...
def getaddresses(fieldvalues): ...
def formatdate(timeval=..., localtime=..., usegmt=...): ...
def format_datetime(dt, usegmt=...): ...
def make_msgid(idstring=..., domain=...): ...
def parsedate_to_datetime(data): ...
def parseaddr(addr): ...
def unquote(str): ...
def decode_rfc2231(s): ...
def encode_rfc2231(s, charset=..., language=...): ...
def decode_params(params): ...
def collapse_rfc2231_value(value, errors=..., fallback_charset=...): ...
